c  
c
c                                  SILICIO
C
c Estructura de bandas.
c Densidad de estados total.
c Densidades de estados parciales de los orbitales s y p
c
c Autores:
c
c Pablo Luiseti
c Carolina Crovetto
c
c Materia Condesada - Licenciatura en Fisica 
c Universidad Nacional de Rosario, Argentina.
c       
c--------------------------------------------------------------------------    
     
      program Silicio

c-----Declaracion de variables

      integer l,n,q,r,h,j
      integer f,p,nbin
      parameter (nbin=200)
      real t,y,pi
      real EINF,ESUP,DE,x,c,d,z
      dimension W(3),V(8),g(600,8),OO(3)
      dimension BB(8)
      dimension os(nbin),op(nbin),opx(nbin),opy(nbin),opz(nbin)
      dimension DEN(nbin)
      complex AV(8,8)
      complex EV(8,8)
      real es,ep,sigmass,sigmasp,sigmapp,pipp
      common /param/ es,ep,sigmass,sigmasp,sigmapp,pipp

      pi=4.*atan(1.)

c-----Lee parametros tight-binding
C-----Energias onsite para orbitales s y p
      read(5,*) es,ep
C-----Parametros de hopping ss_sigma, sp_sigma, pp_sigma, pp_pi
      read(5,*) sigmass,sigmasp
      read(5,*) sigmapp,pipp

c-----Apertura de archivos para la salida

      open (1,file='atomo1.dat',status='unknown')
      open (2,file='atomo2.dat',status='unknown')
      open (3,file='totales.dat',status='unknown')
      open (4,file='pxpypz.dat',status='unknown')
      open (7,file='energias.dat',status='unknown')

c-------------------------------------------------------------------------
c
c-----Calculo de la estructura de bandas para un camino particular
c-----El parametro de red se define como a=1.

      write (*,*) 'Calculando la estructura de bandas' 

      t=0

      do j=1,30
        y=j

        select case (j)

CCCCC---De L a Gamma
        case (1:10)
          t=pi*(1-(y-1)/9)
          OO(1)=t
          OO(2)=t
          OO(3)=t

CCCCC---De Gamma a X
        case (11:20)
          t=2*pi*((y-11)/9)
          OO(1)=t
          OO(2)=0
          OO(3)=0

CCCCC---De X a K y a Gamma
        case (21:30)
          t=2*pi*(1-(y-21)/9)
          OO(1)=t
          OO(2)=t
          OO(3)=0

        end select

        CALL ENERGIAS(OO,BB,AV)

CCCC----Escribe las energias en unidad 5

        write(7,*) t,(BB(l),l=8,1,-1)

      end do

c-------------------------------------------------------------------------
c
c Calculo de la densidad de estados total y parciales 
c
c-----intervalos en el espacio k = p
c-----intervalos en las energias = f
c
      p=30
      f=100

c-----Asigna los valores extremos de las energias para el calculo 
c-----de la densidad de estados

      z=0
      do j=1,f
        DEN(j)=0
        do h=1,8
          g(j,h)=0.
        enddo
      enddo

      EINF=-15.
      ESUP=15.

c-----Intervalo de energia para el histograma

      DE= (ESUP - EINF)/f

c-----Calculo de los valores K dentro de la primera zona de Brioullin.
c-----esto se hace tomando como 1 zona de Brillouin a la celda primitiva 
c-----de la red BCC que es la red reciproca de la FCC.
c-----Los vectores K son cl de los primitivos de la BCC, con coeficientes 
c-----discretos entre 0 y 1

      write(*,*)'Calculando la densidad de estados '

      do l=0,p
        do n=0,p
          do q=0,p
            z=z+1
            W(1)= ((2.*pi*(-n+l+q))/p)
            W(2)= ((2.*pi*(n-l+q))/p)
            W(3)= ((2.*pi*(n+l-q))/p)

c-----------llama a la subrutina de Pablo para calcular los ocho autovalores 
c-----------de energia para este valor de K

            CALL ENERGIAS(W,V,EV)

c-----------compara para cada banda en que rango de energias se encuentra, y 
c-----------suma una unidad donde corresponda para calcular la densidad total. 
c-----------Suma con un peso dado por el modulo cuadrado de la proyeccion de 
c-----------la funcion de onda sobre cada orbital, para calcular las parciales

c-----------Cada  autovalor tiene un autovector de 8 componentes complejas
c-----------Estas componentes son la proyeccion de la funcion de onda sobre 
c-----------los orbitales:
c---------1- 1s
c---------2- 1px
c---------3- 1py
c---------4- 1pz 
c---------5- 2s
c---------6- 2px
c---------7- 2py
c---------8- 2pz 

c---------DEN da la densidad total de estados y g da las densidades
c---------parciales para cada uno de los ocho orbitales del Si

            do j=1,f
              do r=1,8
                if((V(r).ge.(EINF+DE*(j-1))).and.
     .             (V(r).lt.(EINF+DE*j))) then
                  DEN(j)=DEN(j)+1
                  do h=1,8 
                    x=CABS(EV(h,r)) 
                    g(j,h)=g(j,h)+x**2
                  enddo
                endif
              enddo
            enddo

          enddo
        enddo
      enddo

c-----Normaliza la densidad de estados 
c-----Guarda el resultado de frecuencias total y parciales en un archivo
c-----Junta las densidades de los orbitales s, px, py, pz para ambos atomos,
c-----asi como tambien calcula la densidad total debido a los orbitales p

      do j=1,f

        c=EINF+DE*(j-1)
        d=EINF+DE*j

        do h=1,8
          g(j,h)=g(j,h)/(z*8)
          if ((h.EQ.1).OR.(h.EQ.5)) then
            os(j)=os(j) + g(j,h)
          else if ((h.EQ.2).OR.(h.EQ.6)) then
            opx(j)=opx(j) + g(j,h)
          else if ((h.EQ.3).OR.(h.EQ.7)) then
            opy(j)=opy(j) + g(j,h)
          else if ((h.EQ.4).OR.(h.EQ.8)) then
            opz(j)=opz(j) + g(j,h)
          endif
        enddo

        op(j)=opx(j)+opy(j)+opz(j)
        DEN(j) = DEN(j)/(z*8)

        write (1,*) c,d,g(j,1),g(j,2),g(j,3),g(j,4)
        write (2,*) c,d,g(j,5),g(j,6),g(j,7),g(j,8)
        write (3,*) c,d,os(j),op(j),DEN(j)
        write (4,*) c,d,opx(j),opy(j),opz(j)
      enddo

      write(*,*)'Numero de puntos k en la primera zona ',int(z)

      end


c___________________________________________________________________________

c-------Subrutina para el calculo de las energias para cada valor de K
c-------Gracias Pablo!!!!!!!

      SUBROUTINE ENERGIAS(W,V,EV)

c ---------------------------------------------
c
c    Tigh Binding Si (fcc)
c    Pablo Luiseti
c
c -----------------------------------------------

      integer ap,al,jl,jp,g
      real W(3),V(8)
      real t
      complex H
      complex M
      complex D(8,8),A(36),EV(8,8)

CCCC--Generar Matriz
      do i=1,8
        do j=1,8

          if(j.le.4) then 
            ap=j
            jp=0
          else
            ap=j-4
            jp=1
          endif

          if(i.le.4) then 
            al=i
            jl=0
          else
            al=i-4
            jl=1
          endif

          D(i,j)=M(ap,jp,al,jl,W)

        enddo
      enddo

CCCC--Diagonaliza Matriz

      CALL DIAHER(D,8,V,EV,A,36)

      end

c___________________________________________________________________________

CCCC-----Funcion que da el hopping entre un orbital O1 y otro orbital O2
CCCC-----siendo r un versor en la direccion del vector diferencia

      complex function H(O1,r,O2)
      integer O1,O2
      real r(3),fp
      real es,ep,sigmass,sigmasp,sigmapp,pipp
      common /param/ es,ep,sigmass,sigmasp,sigmapp,pipp

C Parametros tight-binding originales para Silicio
C Energias onsite s y p
c      es=-5.25
c      ep=1.20
C Parametros de hopping ss_sigma, sp_sigma, pp_sigma, pp_pi
c      sigmass=-1.9375
c      sigmasp=1.745
c      sigmapp=3.050
c      pipp=-1.075
 
      if ((r(1).eq.0).and.(r(2).eq.0).and.(r(3).eq.0)) then

        if (O1.eq.O2) then
          if (O1.eq.1) then
            H=es
          else
            H=ep
          endif
        else
          H=0
        endif

      else

        if(O1.eq.1) then

          select case (O2)
          case (1)
            H=sigmass
          case (2)
            H=r(1)*sigmasp
          case (3)
            H=r(2)*sigmasp
          case (4)
            H=r(3)*sigmasp
          end select

        else 

          if (O2.eq.1) then
            H=(-1)*r(O1-1)*sigmasp
          else
            H=r(O1-1)*r(O2-1)*sigmapp+fp(O1,r,O2)*pipp
          endif

        endif

      endif

      end

c___________________________________________________________________________

CCCC---Funcion complementaria de H: da el factor de hopping paralelo entre 
CCCC---dos orbiatles Pi	

      real function fp(O1,r,O2)
      integer O1,O2,delta
      real r(3)

      fp=0

      do k=1,3
        fp=fp+(delta(O1-1,k)-r(O1-1)*r(k))*(delta(O2-1,k)-r(O2-1)*r(k))
      end do

      return
      end

c___________________________________________________________________________

      integer function delta(i,j)
      integer i,j

      if (i.eq.j) then
        delta=1
      else
        delta=0
      endif

      return
      end

c___________________________________________________________________________

CCCCC----Funcion que da el vector diferencia entre el atomo (0,j) 
CCCCC----y el (m,jp)=(0..4)    

      real function R(m,j,jp,c)
      integer m,j,jp,c
      real aux 
      real d(3)
      real a(4,3)
      data d,a/3*.25,2*.5,2*0,.5,0,.5,2*0,2*.5,0/

      if (jp.eq.0) then
        aux=-1.
      else
        aux=1.
      endif

      if(m.eq.0) then
        R=0.
      else
        R=aux*(a(m,c)-d(c))
      endif

      end

c___________________________________________________________________________

CCCC----Genera la suma sum(exp[ I K.(R+dj-djp) ] < 0 jp alfap | H | m l alfa >)
 
      complex function M(ap,jp,a,j,K)
      integer ap,jp,a,j
      integer i,l
      real K(3),modul,R
      real aux(5,3)
      real aux2(3)
      complex H,G

      do i=1,5
        do l=1,3
          if(modul(R(i-1,j,jp,1),R(i-1,j,jp,2),R(i-1,j,jp,3)).eq.0) then 
            aux(i,l)=0.
          else
            aux(i,l)=R(i-1,j,jp,l)/modul(R(i-1,j,jp,1),R(i-1,j,jp,2), 
     .               R(i-1,j,jp,3))
          end if
        enddo
      enddo

      if(j.eq.jp) then
        aux2(1)=aux(1,1)
        aux2(2)=aux(1,2)
        aux2(3)=aux(1,3)
        M=exp((0,.1)*(K(1)*R(0,j,jp,1)+K(2)*R(0,j,jp,2)+K(3)*
     .         R(0,j,jp,3)))*H(ap,aux2,a)
      else
        M=(0.,0.)
        do i=1,4
          aux2(1)=aux(i+1,1)
          aux2(2)=aux(i+1,2)
          aux2(3)=aux(i+1,3)
          M=M+cexp((0,1.)*(K(1)*R(i,j,jp,1)+K(2)*R(i,j,jp,2)+K(3)*
     .              R(i,j,jp,3)))*H(ap,aux2,a)
        enddo
      end if

      end

c___________________________________________________________________________

      real function modul(v1,v2,v3)
      real v1,v2,v3
      real a
      modul=sqrt(v1**2+v2**2+v3**2)
      return
      end

c___________________________________________________________________________

C  DIAGNALIZATION OF HERMITEAN MATRIX. SIMPLE PRECISION.                 
C                                                                        
      SUBROUTINE DIAHER (D,N,V,EV,A,NN)                                  
C VERTION FOR FONCAPM -> PHONON DYNAMICS WITH SYMMETRY 
C  AUXD= AUXILIAR MATRIX WICH CONTAINS "D" 
C  D = HERMITEAN MATRIX TO BE DIAGONALIZED                               
C  N = DIMENSION OF D                                                    
C  V = EIGENVALUES                                                       
C  EV(I,J) = EIGENVECTORS (I = COMPONENT, J = EIGENVECTOR INDEX)         
C  A = AUXILIARY VECTOR OF DIMENSION N*(N+1)/2. THE UPPER PART OF THE    
C      MATRIX D WILL BE TRANSFERRED TO A, NUMBERED BY COLUMNS            
C  NN = N*(N+1)/2    
      COMPLEX D(8,8),EV(8,8),A(36),AUXEV(5000)                      
      DIMENSION V(8)                         
      K=0                                                                
      DO 1 I=1,N                                                         
      DO 1 J=1,I                                                         
      K=K+1                                                              
    1 A(K)=D(J,I)                                                        
      CALL CEIGEN(A,AUXEV,N,0)                                              
      DO 2 I=1,N                                                         
      K=(I*(I+1))/2                                                      
    2 V(I)=REAL(A(K))  
C---------VERSION FOR FONCAM----------------- 
      L=0                                                   
      DO I=1,N 
        DO J=1,N 
          L=L+1 
          EV(J,I)=AUXEV(L) 
        ENDDO 
      ENDDO 
C------------------------------------------- 
      RETURN                                                             
      END                                                                
      
      SUBROUTINE CEIGEN(A,R,N,MV)                                        
      COMPLEX A(1), R(1),Y,Z,SINX,SINCS                                  
C     GENERATE IDENTITY MATRIX                                           
      IF(MV-1) 10,25,10                                                  
  10  IQ=-N                                                              
      DO 20 J=1,N                                                        
      IQ=IQ+N                                                            
      DO 20 I=1,N                                                        
      IJ=IQ+I                                                            
      R(IJ)=(0.,0.)                                                      
      IF(I-J) 20,15,20                                                   
  15  R(IJ)=(1.,0.)                                                      
  20  CONTINUE                                                           
C     COMPUTE INITIAL AND FINAL NORMS (ANORM AND ANORMX)                 
  25  ANORM=0.                                                           
      DO 35 I=1,N                                                        
      DO 35 J=I,N                                                        
      IF(1-J) 30,35,30                                                   
  30  IA=I+(J*J-J)/2                                                     
      Y=A(IA)                                                            
      ANORM=ANORM+ CABS(Y*Y)                                             
  35  CONTINUE                                                           
      IF(ANORM) 165,165,40                                               
  40  ANORM=1.4142* SQRT(ANORM)                                           
      ANORMX=ANORM*1.E-6 / FLOAT(N)                                      
C      INITIALIZE INDICATORS AND COMPUTE THRESHOLD, THR                  
      IND=0                                                              
      THR=ANORM                                                          
  45  THR=THR/ FLOAT(N)                                                  
  50  L=1                                                                
  55  M=L+1                                                              
C    KOMPUTE SIN AND COS                                                 
  60  MQ=(M*M-M)/2                                                       
      LQ=(L*L-L)/2                                                       
      LM=L+MQ                                                            
  62  IF( CABS(A(LM))-THR) 130,65,65                                     
  65  IND=1                                                              
      LL=L+LQ                                                            
      MM=M+MQ                                                            
      X=0.5*( REAL(A(LL))- REAL(A(MM)))                                  
      Y=A(LM)                                                            
      Y = -Y/ SQRT( CABS(Y*Y) +X*X)                                      
      IF(X) 70,75,75                                                     
  70  Y=-Y                                                               
  75  IF( CABS(Y*Y) .LE.1.0) GO TO 76                                    
      T2 = 2.                                                            
      SINX=Y/ SQRT(T2)                                                   
      GO TO 77                                                           
c-------------------------------------------------   
  76  SINX=Y/ SQRT(2.0*(1.+  SQRT(abs(1.0- CABS(Y*Y))))) 
c-------------------------------------------------   
  77  SINX2= CABS(SINX*SINX)                                             
  78  COSX= SQRT(1.-SINX2) 
      COSX2=COSX*COSX                                                    
  79  SINX2= CABS(SINX*SINX)                                             
      SINCS=SINX*COSX                                                    
C     ROTATE L AND M COLUMNS                                             
      ILQ=N*(L-1)                                                        
      IMQ=N*(M-1)                                                        
      DO 125 I=1,N                                                       
      IQ=(I*I-I)/2                                                       
      IF(I-L) 80,115,85                                                  
  80  IM=I+MQ                                                            
      IL=I+LQ                                                          
      Y=A(IL)*COSX-A(IM)* CONJG(SINX)                                    
      Z=A(IL)*SINX+A(IM)*COSX                                            
      GO TO 100                                                          
  85  IF(I-M) 90,115,95                                                  
  90  IM=I+MQ                                                            
      IL=L+IQ                                                            
      Y=A(IL)*COSX- CONJG(A(IM))*SINX                                    
      Z= CONJG(A(IL))*SINX+A(IM)*COSX                                    
      GO TO 100                                                          
  95  IM=M+IQ                                                            
      IL=L+IQ                                                            
      Y=A(IL)*COSX-A(IM)*SINX                                            
      Z=A(IL)* CONJG(SINX)+A(IM)*COSX                                    
 100  A(IL)=Y                                                            
      A(IM)=Z                                                            
  115 IF(MV-1) 120,125,120                                               
  120 ILR=ILQ+I                                                          
      IMR=IMQ+I                                                          
      Y=R(ILR)*COSX-R(IMR)* CONJG(SINX)                                  
      R(IMR)=R(ILR)*SINX+R(IMR)*COSX                                     
      R(ILR)=Y                                                           
  125 CONTINUE                                                           
      X=2.* REAL(SINCS*CONJG(A(LM)))                                     
      Y=A(LL)*COSX2+A(MM)*SINX2-X                                        
      Z=A(LL)*SINX2+A(MM)*COSX2+X                                        
      A(LM)=(A(LL)-A(MM))*SINCS+A(LM)*COSX2-SINX*SINX* CONJG(A(LM))      
      A(LL)=Y                                                            
      A(MM)=Z                                                            
C    TEST  FOR COMPLETION                                                
C     TEST FOR M=LAST COLUMN                                             
  130  IF(M-N) 135,140,135                                               
  135  M=M+1                                                             
      GO TO 60                                                           
C    TEST FOR  L=SECOND FROM LAST COLUMN                                 
  140  IF(L-(N-1)) 145,150,145                                           
  145 L=L+1                                                              
      GO TO 55                                                           
  150  IF(IND-1) 160,155,160                                             
  155  IND=0                                                             
      GO TO 50                                                           
C    COMPARE THRESHOLD WIH FINAL NORM                                    
  160  IF(THR-ANORMX)165,165,45                                          
C    SORT EIGENVALUES AND EIGENVECTORS                                   
  165  IQ=-N                                                             
      DO 185 I=1,N                                                       
      IQ=IQ+N                                                            
      LL=I+(I*I-I)/2                                                     
      JQ=N*(I-2)                                                         
      DO 185 J=I,N                                                       
      JQ=JQ+N                                                            
      MM=J+(J*J-J)/2                                                     
      IF( REAL(A(LL))- REAL(A(MM))) 170,185,185                          
  170  Y=A(LL)                                                           
      A(LL)=A(MM)                                                        
      A(MM)=Y                                                            
      IF(MV-1) 175,185,175                                               
  175 DO 180 K=1,N                                                       
      ILR=IQ+K                                                           
      IMR=JQ+K                                                           
      Y=R(ILR)                                                           
      R(ILR)=R(IMR)                                                      
  180 R(IMR)=Y                                                           
  185 CONTINUE                                                           
      RETURN                                                             
       END                                                               
